@@ -470,44 +470,43 @@ pub fn gen_bitwise(spec: &Spec) -> proc_macro2::TokenStream {
 
     quote! {
         pub mod bitwise {
-            pub struct #name(#ty);
+            pub struct #name;
 
-            impl ::crcany::crc::v2::Crc for #name {
+            impl #name {
+                pub fn new() -> ::crcany::crc::Computer<Self> {
+                    ::crcany::crc::Computer::new(#name)
+                }
+            }
+
+            impl ::crcany::crc::Crc for #name {
                 type Int = #ty;
 
-                fn new() -> Self {
-                    #name(#init)
+                fn init(&self) -> Self::Int {
+                    #init
                 }
 
-                fn add_bytes(&mut self, mut data: &[u8]) {
+                fn add_bytes(&self, mut crc: Self::Int, mut data: &[u8]) -> Self::Int {
                     if data.is_empty() {
-                        return;
+                        return crc;
                     }
 
-                    let mut crc = self.0;
-
                     #xorin
                     #revin
                     #body
 
-                    self.0 = crc;
-                }
-
-                fn to_inner(&self) -> Self::Int {
-                    self.0
+                    crc
                 }
             }
 
             #[cfg(test)]
             mod test {
-                use ::crcany::crc::v2::Crc;
                 use super::*;
 
                 #[test]
                 fn #simple_test_name() {
                     let mut crc = #name::new();
                     crc.add_bytes(b"123456789");
-                    assert_eq!(#check, crc.to_inner());
+                    assert_eq!(#check, crc.into_inner());
                 }
 
                 #[test]
@@ -518,14 +517,14 @@ pub fn gen_bitwise(spec: &Spec) -> proc_macro2::TokenStream {
                     crc.add_bytes(b"");
                     crc.add_bytes(b"567");
                     crc.add_bytes(b"89");
-                    assert_eq!(#check, crc.to_inner());
+                    assert_eq!(#check, crc.into_inner());
                 }
 
                 #[test]
                 fn #empty_test_name() {
                     let mut crc = #name::new();
                     crc.add_bytes(b"");
-                    assert_eq!(#init, crc.to_inner());
+                    assert_eq!(#init, crc.into_inner());
                 }
             }
         }
@@ -533,7 +532,6 @@ pub fn gen_bitwise(spec: &Spec) -> proc_macro2::TokenStream {
 }
 
 pub fn gen_bytewise_table(spec: &Spec) -> proc_macro2::TokenStream {
-    use crate::crc::FromSpec;
     let bytewise = BytewiseModel::from_spec(spec.clone());
 
     let (ty, _) = get_ty(spec);
@@ -636,44 +634,43 @@ pub fn gen_bytewise(spec: &Spec) -> proc_macro2::TokenStream {
 
     quote! {
         pub mod bytewise {
-            pub struct #name(#ty);
+            pub struct #name;
 
-            impl ::crcany::crc::v2::Crc for #name {
+            impl #name {
+                pub fn new() -> ::crcany::crc::Computer<Self> {
+                    ::crcany::crc::Computer::new(#name)
+                }
+            }
+
+            impl ::crcany::crc::Crc for #name {
                 type Int = #ty;
 
-                fn new() -> Self {
-                    #name(#init)
+                fn init(&self) -> Self::Int {
+                    #init
                 }
 
-                fn add_bytes(&mut self, mut data: &[u8]) {
+                fn add_bytes(&self, mut crc: Self::Int, mut data: &[u8]) -> Self::Int {
                     if data.is_empty() {
-                        return;
+                        return crc;
                     }
 
-                    let mut crc = self.0;
-
                     #rev
                     #body
                     #rev
 
-                    self.0 = crc;
-                }
-
-                fn to_inner(&self) -> Self::Int {
-                    self.0
+                    crc
                 }
             }
 
             #[cfg(test)]
             mod test {
-                use ::crcany::crc::v2::Crc;
                 use super::*;
 
                 #[test]
                 fn #simple_test_name() {
                     let mut crc = #name::new();
                     crc.add_bytes(b"123456789");
-                    assert_eq!(#check, crc.to_inner());
+                    assert_eq!(#check, crc.into_inner());
                 }
 
                 #[test]
@@ -684,14 +681,14 @@ pub fn gen_bytewise(spec: &Spec) -> proc_macro2::TokenStream {
                     crc.add_bytes(b"");
                     crc.add_bytes(b"567");
                     crc.add_bytes(b"89");
-                    assert_eq!(#check, crc.to_inner());
+                    assert_eq!(#check, crc.into_inner());
                 }
 
                 #[test]
                 fn #empty_test_name() {
                     let mut crc = #name::new();
                     crc.add_bytes(b"");
-                    assert_eq!(#init, crc.to_inner());
+                    assert_eq!(#init, crc.into_inner());
                 }
             }
         }
@@ -699,7 +696,6 @@ pub fn gen_bytewise(spec: &Spec) -> proc_macro2::TokenStream {
 }
 
 pub fn gen_wordwise_table(spec: &Spec) -> proc_macro2::TokenStream {
-    use crate::crc::FromSpec;
     let wordwise =
         WordwiseModel::<byteorder::NativeEndian, { usize::BITS as _ }>::from_spec(spec.clone());
 
@@ -854,33 +850,33 @@ pub fn gen_wordwise(spec: &Spec) -> proc_macro2::TokenStream {
 
     quote! {
         pub mod wordwise {
-            pub struct #name(#ty);
+            pub struct #name;
 
-            impl ::crcany::crc::v2::Crc for #name {
+            impl #name {
+                pub fn new() -> ::crcany::crc::Computer<Self> {
+                    ::crcany::crc::Computer::new(#name)
+                }
+            }
+
+            impl ::crcany::crc::Crc for #name {
                 type Int = #ty;
 
-                fn new() -> Self {
-                    #name(#init)
+                fn init(&self) -> Self::Int {
+                    #init
                 }
 
-                fn add_bytes(&mut self, mut data: &[u8]) {
+                fn add_bytes(&self, mut crc: Self::Int, mut data: &[u8]) -> Self::Int {
                     if data.is_empty() {
-                        return;
+                        return crc;
                     }
 
-                    let mut crc = self.0;
-
                     #maybe_rev
                     #initial_bytes
                     #word_body
                     #final_bytes
                     #maybe_rev
 
-                    self.0 = crc;
-                }
-
-                fn to_inner(&self) -> Self::Int {
-                    self.0
+                    crc
                 }
             }
 
@@ -888,7 +884,6 @@ pub fn gen_wordwise(spec: &Spec) -> proc_macro2::TokenStream {
 
             #[cfg(test)]
             mod test {
-                use ::crcany::crc::v2::Crc;
                 use super::*;
 
                 const INPUT: &[u8] = b"123456789123456789123456789123456789123456789123456789123456789123456789123456789";
@@ -900,7 +895,8 @@ pub fn gen_wordwise(spec: &Spec) -> proc_macro2::TokenStream {
                         let end = start + 9;
                         let mut crc = #name::new();
                         crc.add_bytes(&INPUT[start..end]);
-                        assert_eq!(#check, crc.to_inner(), "failure on permutation {} - 0b{:b} vs 0b{:b}", i, crc.to_inner(), #check);
+                        let res = crc.into_inner();
+                        assert_eq!(#check, res, "failure on permutation {} - 0b{:b} vs 0b{:b}", i, res, #check);
                     }
                 }
 
@@ -912,14 +908,14 @@ pub fn gen_wordwise(spec: &Spec) -> proc_macro2::TokenStream {
                     crc.add_bytes(b"");
                     crc.add_bytes(b"567");
                     crc.add_bytes(b"89");
-                    assert_eq!(#check, crc.to_inner());
+                    assert_eq!(#check, crc.into_inner());
                 }
 
                 #[test]
                 fn #empty_test_name() {
                     let mut crc = #name::new();
                     crc.add_bytes(b"");
-                    assert_eq!(#init, crc.to_inner());
+                    assert_eq!(#init, crc.into_inner());
                 }
             }
         }
