@@ -10,7 +10,7 @@ pub use combining::*;
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::crc::{Crc, FromSpec};
+    use crate::crc::Computer;
     use crate::spec::Spec;
 
     fn kermit() -> Spec {
@@ -33,7 +33,7 @@ mod test {
 
         let input = b"123456789";
 
-        let output = model.crc(0, input.iter().copied());
+        let output = Computer::crc(model, &input[..]);
 
         assert_eq!(0x2189, output);
     }
@@ -44,7 +44,7 @@ mod test {
 
         let input = b"123456789";
 
-        let output = model.crc(0, input.iter().copied());
+        let output = Computer::crc(model, &input[..]);
 
         assert_eq!(0x2189, output);
     }
@@ -56,7 +56,7 @@ mod test {
 
         let input = b"123456789";
 
-        let output = model.crc(0, input.iter().copied());
+        let output = Computer::crc(model, &input[..]);
 
         assert_eq!(0x2189, output);
     }
